@@ -302,7 +302,7 @@ def validate_signature_fields(sig, mandatory_fields=[b'v', b'a', b'b', b'bh', b'
     try:
         str(sig[b'd'], 'ascii')
         # No specials, which is close enough
-        if re.findall(b"[\(\)<>\[\]:;@\\,]", sig[b'd']):
+        if re.findall(rb"[\(\)<>\[\]:;@\\,]", sig[b'd']):
             raise ValidationError("d= value is not valid (%s)" % sig[b'd'])
     except UnicodeDecodeError as e:
         # Not an ASCII domain
@@ -468,6 +468,8 @@ def evaluate_pk(name, s):
               pk = nacl.signing.VerifyKey(pub[b'p'], encoder=nacl.encoding.Base64Encoder)
           except NameError:
               raise NaClNotFoundError('pynacl module required for ed25519 signing, see README.md')
+          except nacl.exceptions.ValueError as e:
+              raise KeyFormatError("could not parse ed25519 public key (%s): %s" % (pub[b'p'],e))
           keysize = 256
           ktag = b'ed25519'
   except KeyError:
@@ -477,9 +479,9 @@ def evaluate_pk(name, s):
           pk = parse_public_key(base64.b64decode(pub[b'p']))
           keysize = bitsize(pk['modulus'])
       except KeyError:
-          raise KeyFormatError("incomplete public key: %s" % s)
+          raise KeyFormatError("incomplete RSA public key: %s" % s)
       except (TypeError,UnparsableKeyError) as e:
-          raise KeyFormatError("could not parse public key (%s): %s" % (pub[b'p'],e))
+          raise KeyFormatError("could not parse RSA public key (%s): %s" % (pub[b'p'],e))
       ktag = b'rsa'
   if pub[b'k'] != b'rsa' and pub[b'k'] != b'ed25519':
       raise KeyFormatError('unknown algorithm in k= tag: {0}'.format(pub[b'k']))
@@ -800,7 +802,7 @@ class DomainSigner(object):
     except binascii.Error as e:
       self.logger.error('KeyFormatError: {0}'.format(e))
       return False
-    except dns.exception.Timeout as e:
+    except DnsTimeoutError as e:
       self.logger.error('DnsTimeoutError: Domain: {0} Selector: {1} Error message: {2}'.format(
           sig[b'd'], sig[b's'], e))
       return False
@@ -1293,7 +1295,9 @@ class ARC(DomainSigner):
     # we can't use the AMS provided above, as it's already been canonicalized relaxed
     # for use in validating the AS.  However the AMS is included in the AMS itself,
     # and this can use simple canonicalization
-    raw_ams_header = [(x, y) for (x, y) in self.headers if x.lower() == b'arc-message-signature'][0]
+    raw_ams_header = [
+       (x, y) for (x, y) in self.headers if x.lower() == b'arc-message-signature' and b" i="+sig[b'i']+b";" in y.lower()
+    ][0]
 
     # Only relaxed canonicalization used by ARC
     if b'c' not in sig:
